#!/usr/bin/env python
# encoding: utf-8
# John O'Meara, 2006
# Thomas Nagy 2009-2018 (ita)

"""
The **bison** program is a code generator which creates C or C++ files.
The generated files are compiled into object files.
"""

from waflib import Task
from waflib.TaskGen import extension

class bison(Task.Task):
	"""Compiles bison files"""
	color   = 'BLUE'
	run_str = '${BISON} ${BISONFLAGS} ${SRC[0].abspath()} -o ${TGT[0].name}'
	ext_out = ['.h'] # just to make sure

@extension('.y', '.yc', '.yy')
def big_bison(self, node):
	"""
	Creates a bison task, which must be executed from the directory of the output file.
	"""
	has_h = '-d' in self.env.BISONFLAGS

	outs = []
	if node.name.endswith('.yc'):
		outs.append(node.change_ext('.tab.cc'))
		if has_h:
			outs.append(node.change_ext('.tab.hh'))
	else:
		outs.append(node.change_ext('.tab.c'))
		if has_h:
			outs.append(node.change_ext('.tab.h'))

	tsk = self.create_task('bison', node, outs)
	tsk.cwd = node.parent.get_bld()

	# and the c/cxx file must be compiled too
	self.source.append(outs[0])

def configure(conf):
	"""
	Detects the *bison* program
	"""
	conf.find_program('bison', var='BISON')
	conf.env.BISONFLAGS = ['-d']




#! /usr/bin/env python
# encoding: utf-8
# imported from samba

"""
Compiler definition for irix/MIPSpro cc compiler
"""

from waflib import Errors
from waflib.Tools import ccroot, ar
from waflib.Configure import conf

@conf
def find_irixcc(conf):
	v = conf.env
	cc = None
	if v.CC:
		cc = v.CC
	elif 'CC' in conf.environ:
		cc = conf.environ['CC']
	if not cc:
		cc = conf.find_program('cc', var='CC')
	if not cc:
		conf.fatal('irixcc was not found')

	try:
		conf.cmd_and_log(cc + ['-version'])
	except Errors.WafError:
		conf.fatal('%r -version could not be executed' % cc)

	v.CC = cc
	v.CC_NAME = 'irix'

@conf
def irixcc_common_flags(conf):
	v = conf.env

	v.CC_SRC_F            = ''
	v.CC_TGT_F            = ['-c', '-o']
	v.CPPPATH_ST          = '-I%s'
	v.DEFINES_ST          = '-D%s'

	if not v.LINK_CC:
		v.LINK_CC = v.CC

	v.CCLNK_SRC_F         = ''
	v.CCLNK_TGT_F         = ['-o']

	v.LIB_ST              = '-l%s' # template for adding libs
	v.LIBPATH_ST          = '-L%s' # template for adding libpaths
	v.STLIB_ST            = '-l%s'
	v.STLIBPATH_ST        = '-L%s'

	v.cprogram_PATTERN    = '%s'
	v.cshlib_PATTERN      = 'lib%s.so'
	v.cstlib_PATTERN      = 'lib%s.a'

def configure(conf):
	conf.find_irixcc()
	conf.find_cpp()
	conf.find_ar()
	conf.irixcc_common_flags()
	conf.cc_load_tools()
	conf.cc_add_flags()
	conf.link_add_flags()




#! /usr/bin/env python
# encoding: UTF-8
# Petar Forai
# Thomas Nagy 2008-2010 (ita)

import re
from waflib import Task, Logs
from waflib.TaskGen import extension, feature, after_method
from waflib.Configure import conf
from waflib.Tools import c_preproc

"""
tasks have to be added dynamically:
- swig interface files may be created at runtime
- the module name may be unknown in advance
"""

SWIG_EXTS = ['.swig', '.i']

re_module = re.compile(r'%module(?:\s*\(.*\))?\s+(.+)', re.M)

re_1 = re.compile(r'^%module.*?\s+([\w]+)\s*?$', re.M)
re_2 = re.compile(r'[#%](?:include|import(?:\(module=".*"\))+|python(?:begin|code)) [<"](.*)[">]', re.M)

class swig(Task.Task):
	color   = 'BLUE'
	run_str = '${SWIG} ${SWIGFLAGS} ${SWIGPATH_ST:INCPATHS} ${SWIGDEF_ST:DEFINES} ${SRC}'
	ext_out = ['.h'] # might produce .h files although it is not mandatory
	vars = ['SWIG_VERSION', 'SWIGDEPS']

	def runnable_status(self):
		for t in self.run_after:
			if not t.hasrun:
				return Task.ASK_LATER

		if not getattr(self, 'init_outputs', None):
			self.init_outputs = True
			if not getattr(self, 'module', None):
				# search the module name
				txt = self.inputs[0].read()
				m = re_module.search(txt)
				if not m:
					raise ValueError("could not find the swig module name")
				self.module = m.group(1)

			swig_c(self)

			# add the language-specific output files as nodes
			# call funs in the dict swig_langs
			for x in self.env['SWIGFLAGS']:
				# obtain the language
				x = x[1:]
				try:
					fun = swig_langs[x]
				except KeyError:
					pass
				else:
					fun(self)

		return super(swig, self).runnable_status()

	def scan(self):
		"scan for swig dependencies, climb the .i files"
		lst_src = []

		seen = []
		missing = []
		to_see = [self.inputs[0]]

		while to_see:
			node = to_see.pop(0)
			if node in seen:
				continue
			seen.append(node)
			lst_src.append(node)

			# read the file
			code = node.read()
			code = c_preproc.re_nl.sub('', code)
			code = c_preproc.re_cpp.sub(c_preproc.repl, code)

			# find .i files and project headers
			names = re_2.findall(code)
			for n in names:
				for d in self.generator.includes_nodes + [node.parent]:
					u = d.find_resource(n)
					if u:
						to_see.append(u)
						break
				else:
					missing.append(n)
		return (lst_src, missing)

# provide additional language processing
swig_langs = {}
def swigf(fun):
	swig_langs[fun.__name__.replace('swig_', '')] = fun
	return fun
swig.swigf = swigf

def swig_c(self):
	ext = '.swigwrap_%d.c' % self.generator.idx
	flags = self.env['SWIGFLAGS']
	if '-c++' in flags:
		ext += 'xx'
	out_node = self.inputs[0].parent.find_or_declare(self.module + ext)

	if '-c++' in flags:
		c_tsk = self.generator.cxx_hook(out_node)
	else:
		c_tsk = self.generator.c_hook(out_node)

	c_tsk.set_run_after(self)

	# transfer weights from swig task to c task
	if getattr(self, 'weight', None):
		c_tsk.weight = self.weight
	if getattr(self, 'tree_weight', None):
		c_tsk.tree_weight = self.tree_weight

	try:
		self.more_tasks.append(c_tsk)
	except AttributeError:
		self.more_tasks = [c_tsk]

	try:
		ltask = self.generator.link_task
	except AttributeError:
		pass
	else:
		ltask.set_run_after(c_tsk)
		# setting input nodes does not declare the build order
		# because the build already started, but it sets
		# the dependency to enable rebuilds
		ltask.inputs.append(c_tsk.outputs[0])

	self.outputs.append(out_node)

	if not '-o' in self.env['SWIGFLAGS']:
		self.env.append_value('SWIGFLAGS', ['-o', self.outputs[0].abspath()])

@swigf
def swig_python(tsk):
	node = tsk.inputs[0].parent
	if tsk.outdir:
		node = tsk.outdir
	tsk.set_outputs(node.find_or_declare(tsk.module+'.py'))

@swigf
def swig_ocaml(tsk):
	node = tsk.inputs[0].parent
	if tsk.outdir:
		node = tsk.outdir
	tsk.set_outputs(node.find_or_declare(tsk.module+'.ml'))
	tsk.set_outputs(node.find_or_declare(tsk.module+'.mli'))

@extension(*SWIG_EXTS)
def i_file(self, node):
	# the task instance
	tsk = self.create_task('swig')
	tsk.set_inputs(node)
	tsk.module = getattr(self, 'swig_module', None)

	flags = self.to_list(getattr(self, 'swig_flags', []))
	tsk.env.append_value('SWIGFLAGS', flags)

	tsk.outdir = None
	if '-outdir' in flags:
		outdir = flags[flags.index('-outdir')+1]
		outdir = tsk.generator.bld.bldnode.make_node(outdir)
		outdir.mkdir()
		tsk.outdir = outdir

@feature('c', 'cxx', 'd', 'fc', 'asm')
@after_method('apply_link', 'process_source')
def enforce_swig_before_link(self):
	try:
		link_task = self.link_task
	except AttributeError:
		pass
	else:
		for x in self.tasks:
			if x.__class__.__name__ == 'swig':
				link_task.run_after.add(x)

@conf
def check_swig_version(conf, minver=None):
	"""
	Check if the swig tool is found matching a given minimum version.
	minver should be a tuple, eg. to check for swig >= 1.3.28 pass (1,3,28) as minver.

	If successful, SWIG_VERSION is defined as 'MAJOR.MINOR'
	(eg. '1.3') of the actual swig version found.

	:param minver: minimum version
	:type minver: tuple of int
	:return: swig version
	:rtype: tuple of int
	"""
	assert minver is None or isinstance(minver, tuple)
	swigbin = conf.env['SWIG']
	if not swigbin:
		conf.fatal('could not find the swig executable')

	# Get swig version string
	cmd = swigbin + ['-version']
	Logs.debug('swig: Running swig command %r', cmd)
	reg_swig = re.compile(r'SWIG Version\s(.*)', re.M)
	swig_out = conf.cmd_and_log(cmd)
	swigver_tuple = tuple([int(s) for s in reg_swig.findall(swig_out)[0].split('.')])

	# Compare swig version with the minimum required
	result = (minver is None) or (swigver_tuple >= minver)

	if result:
		# Define useful environment variables
		swigver = '.'.join([str(x) for x in swigver_tuple[:2]])
		conf.env['SWIG_VERSION'] = swigver

	# Feedback
	swigver_full = '.'.join(map(str, swigver_tuple[:3]))
	if minver is None:
		conf.msg('Checking for swig version', swigver_full)
	else:
		minver_str = '.'.join(map(str, minver))
		conf.msg('Checking for swig version >= %s' % (minver_str,), swigver_full, color=result and 'GREEN' or 'YELLOW')

	if not result:
		conf.fatal('The swig version is too old, expecting %r' % (minver,))

	return swigver_tuple

def configure(conf):
	conf.find_program('swig', var='SWIG')
	conf.env.SWIGPATH_ST = '-I%s'
	conf.env.SWIGDEF_ST = '-D%s'




#!/usr/bin/env python
# encoding: utf-8
# andersg at 0x63.nu 2007
# Thomas Nagy 2016-2018 (ita)

"""
Support for Perl extensions. A C/C++ compiler is required::

	def options(opt):
		opt.load('compiler_c perl')
	def configure(conf):
		conf.load('compiler_c perl')
		conf.check_perl_version((5,6,0))
		conf.check_perl_ext_devel()
		conf.check_perl_module('Cairo')
		conf.check_perl_module('Devel::PPPort 4.89')
	def build(bld):
		bld(
			features     = 'c cshlib perlext',
			source       = 'Mytest.xs',
			target       = 'Mytest',
			install_path = '${ARCHDIR_PERL}/auto')
		bld.install_files('${ARCHDIR_PERL}', 'Mytest.pm')
"""

import os
from waflib import Task, Options, Utils, Errors
from waflib.Configure import conf
from waflib.TaskGen import extension, feature, before_method

@before_method('apply_incpaths', 'apply_link', 'propagate_uselib_vars')
@feature('perlext')
def init_perlext(self):
	"""
	Change the values of *cshlib_PATTERN* and *cxxshlib_PATTERN* to remove the
	*lib* prefix from library names.
	"""
	self.uselib = self.to_list(getattr(self, 'uselib', []))
	if not 'PERLEXT' in self.uselib:
		self.uselib.append('PERLEXT')
	self.env.cshlib_PATTERN = self.env.cxxshlib_PATTERN = self.env.perlext_PATTERN

@extension('.xs')
def xsubpp_file(self, node):
	"""
	Create :py:class:`waflib.Tools.perl.xsubpp` tasks to process *.xs* files
	"""
	outnode = node.change_ext('.c')
	self.create_task('xsubpp', node, outnode)
	self.source.append(outnode)

class xsubpp(Task.Task):
	"""
	Process *.xs* files
	"""
	run_str = '${PERL} ${XSUBPP} -noprototypes -typemap ${EXTUTILS_TYPEMAP} ${SRC} > ${TGT}'
	color   = 'BLUE'
	ext_out = ['.h']

@conf
def check_perl_version(self, minver=None):
	"""
	Check if Perl is installed, and set the variable PERL.
	minver is supposed to be a tuple
	"""
	res = True
	if minver:
		cver = '.'.join(map(str,minver))
	else:
		cver = ''

	self.start_msg('Checking for minimum perl version %s' % cver)

	perl = self.find_program('perl', var='PERL', value=getattr(Options.options, 'perlbinary', None))
	version = self.cmd_and_log(perl + ["-e", 'printf \"%vd\", $^V'])
	if not version:
		res = False
		version = "Unknown"
	elif not minver is None:
		ver = tuple(map(int, version.split(".")))
		if ver < minver:
			res = False

	self.end_msg(version, color=res and 'GREEN' or 'YELLOW')
	return res

@conf
def check_perl_module(self, module):
	"""
	Check if specified perlmodule is installed.

	The minimum version can be specified by specifying it after modulename
	like this::

		def configure(conf):
			conf.check_perl_module("Some::Module 2.92")
	"""
	cmd = self.env.PERL + ['-e', 'use %s' % module]
	self.start_msg('perl module %s' % module)
	try:
		r = self.cmd_and_log(cmd)
	except Errors.WafError:
		self.end_msg(False)
		return None
	self.end_msg(r or True)
	return r

@conf
def check_perl_ext_devel(self):
	"""
	Check for configuration needed to build perl extensions.

	Sets different xxx_PERLEXT variables in the environment.

	Also sets the ARCHDIR_PERL variable useful as installation path,
	which can be overridden by ``--with-perl-archdir`` option.
	"""

	env = self.env
	perl = env.PERL
	if not perl:
		self.fatal('find perl first')

	def cmd_perl_config(s):
		return perl + ['-MConfig', '-e', 'print \"%s\"' % s]
	def cfg_str(cfg):
		return self.cmd_and_log(cmd_perl_config(cfg))
	def cfg_lst(cfg):
		return Utils.to_list(cfg_str(cfg))
	def find_xsubpp():
		for var in ('privlib', 'vendorlib'):
			xsubpp = cfg_lst('$Config{%s}/ExtUtils/xsubpp$Config{exe_ext}' % var)
			if xsubpp and os.path.isfile(xsubpp[0]):
				return xsubpp
		return self.find_program('xsubpp')

	env.LINKFLAGS_PERLEXT = cfg_lst('$Config{lddlflags}')
	env.INCLUDES_PERLEXT = cfg_lst('$Config{archlib}/CORE')
	env.CFLAGS_PERLEXT = cfg_lst('$Config{ccflags} $Config{cccdlflags}')
	env.EXTUTILS_TYPEMAP = cfg_lst('$Config{privlib}/ExtUtils/typemap')
	env.XSUBPP = find_xsubpp()

	if not getattr(Options.options, 'perlarchdir', None):
		env.ARCHDIR_PERL = cfg_str('$Config{sitearch}')
	else:
		env.ARCHDIR_PERL = getattr(Options.options, 'perlarchdir')

	env.perlext_PATTERN = '%s.' + cfg_str('$Config{dlext}')

def options(opt):
	"""
	Add the ``--with-perl-archdir`` and ``--with-perl-binary`` command-line options.
	"""
	opt.add_option('--with-perl-binary', type='string', dest='perlbinary', help = 'Specify alternate perl binary', default=None)
	opt.add_option('--with-perl-archdir', type='string', dest='perlarchdir', help = 'Specify directory where to install arch specific files', default=None)

